/**
 * 
 */
package com.cloudwords.api.client;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import com.cloudwords.api.client.exception.CloudwordsClientException;
import com.cloudwords.api.client.resources.ChangeOrderProjectWorkflow;
import com.cloudwords.api.client.resources.CloudwordsFile;
import com.cloudwords.api.client.resources.CloudwordsLanguageFile;
import com.cloudwords.api.client.resources.CustomerChangeOrder;
import com.cloudwords.api.client.resources.Language;
import com.cloudwords.api.client.resources.PagedProjectQuery;
import com.cloudwords.api.client.resources.PagedResponse;
import com.cloudwords.api.client.resources.Project;
import com.cloudwords.api.client.resources.ProjectWorkflow;
import com.cloudwords.api.client.resources.ProjectWorkflowPhase;
import com.cloudwords.api.client.resources.SourceDocument;
import com.cloudwords.api.client.resources.Task;
import com.cloudwords.api.client.resources.TranslatedDocument;
import com.cloudwords.api.client.resources.Webhook;

/**
 * The API calls that are shared between Customer and Vendor side.
 *
 * @author andrew
 * @since 1.21
 */
public interface BaseCloudwordsAPI {
    /**
     * Get the Cloudwords {@link Project} associated with the unique Cloudwords {@link Project} ID.
     * 
     * @param projectId
     *            ID of the {@link Project} to return
     * @return {@link Project} resource
     * @throws CloudwordsClientException
     */
    public Project getProject(int projectId) throws CloudwordsClientException;
    
    /**
     * Get the list of source documents currently attached to the project.
     * @param projectId The ID of the project
     * @return The list of current source documents
     * @throws CloudwordsClientException
     */
    public List<SourceDocument> getSourceDocuments(int projectId) throws CloudwordsClientException;
    
    /**
     * Get an individual member of the list returned by {@link #getSourceDocuments(int)}.
     * @param projectId The ID of the project
     * @param documentId The ID of the source document
     * @return The metadata for the document
     * @throws CloudwordsClientException
     */
    public SourceDocument getSourceDocument(int projectId, int documentId) throws CloudwordsClientException;
    
    /**
     * Download the content of a source document. Note that if the source is an XLIFF file, it will be available via {@link #downloadSourceDocumentXLIFF(int, int)}
     * instead. In some cases, Cloudwords may extract XLIFF from other types of files, in which case this method will return
     * the original content, and the XLIFF method will return the extracted XLIFF. The metadata for the file returned by this method
     * is available via {@link SourceDocument#getFile()}.
     * @param projectId The ID of the project
     * @param documentId The ID of the document
     * @return An input stream, which the caller is expected to close.
     * @throws CloudwordsClientException
     * @throws IllegalStateException
     * @throws IOException
     */
    public InputStream downloadSourceDocumentFile(int projectId, int documentId) throws CloudwordsClientException, IllegalStateException, IOException;
    
    /**
     * Download the XLIFF of a source document. The metadata is available via {@link SourceDocument#getXliff()}.
     * @param projectId The ID of the project
     * @param documentId The ID of the document
     * @return An input stream, which the caller is expected to close.
     * @throws CloudwordsClientException
     * @throws IllegalStateException
     * @throws IOException
     */
    public InputStream downloadSourceDocumentXLIFF(int projectId, int documentId) throws CloudwordsClientException, IllegalStateException, IOException;
    
    /**
     * Get the {@link Project}'s source material {@link CloudwordsFile} resource for the specified
     * Cloudwords . The metadata will contain a contentPath for downloading the actual binary file.
     * 
     * @param projectId
     *            ID of the {@link Project} to return source materials for
     * @return {@link Project} source {@link CloudwordsFile} resource
     * @throws CloudwordsClientException
     */
    public CloudwordsFile getProjectSource(int projectId) throws CloudwordsClientException;

    /**
     * Download the source material binary for the specified {@link Project} as an
     * {@link InputStream}, provided that a source material exists for the {@link Project}.
     * <p>
     * <b>NOTE:</b> clients utilizing this method must handle the closing of {@link InputStream}
     * after use.
     * </p>
     * 
     * @param projectId
     *            ID of {@link Project} to download source for
     * @return file {@link InputStream} if source file exists, null otherwise
     * @throws CloudwordsClientException
     */
    public InputStream downloadSourceFile(int projectId) throws CloudwordsClientException;

    /**
     * Get the list of {@link Project} reference material metadata for the specified Cloudwords
     * {@link Project}. Each item in the returned list will contain a contentPath for downloading
     * the respective binary file.
     * 
     * @param projectId
     *            ID of the {@link Project} to return reference materials for
     * @return list of {@link Project} reference material {@link CloudwordsFile} resources
     * @throws CloudwordsClientException
     */
    public List<CloudwordsFile> getProjectReferences(int projectId) throws CloudwordsClientException;

    /**
     * Get the specified {@link Project} reference material metadata. The metadata will contain a
     * contentPath for downloading the actual binary file.
     * 
     * @param projectId
     *            ID of the {@link Project} to get reference material for
     * @param documentId
     *            ID of the specific reference material to return
     * @return reference material {@link CloudwordsFile} resource
     * @throws CloudwordsClientException
     */
    public CloudwordsFile getProjectReference(int projectId, int documentId) throws CloudwordsClientException;

    /**
     * Download the specified {@link Project} reference material binary as an {@link InputStream},
     * provided that such reference material exists for the {@link Project}.
     * <p>
     * <b>NOTE:</b> clients utilizing this method must handle the closing of {@link InputStream}
     * after use.
     * </p>
     * 
     * @param projectId
     *            ID of the {@link Project} to download reference material for
     * @param documentId
     *            ID of the specific reference material to download
     * @return file {@link InputStream} if reference file exists, null otherwise
     * @throws CloudwordsClientException
     */
    public InputStream downloadReferenceFile(int projectId, int documentId) throws CloudwordsClientException;

    /**
     * Get the current master {@link Project} translation file metadata, including a contentPath for
     * downloading the actual binary file. The master {@link Project} translation file contains all
     * of the current {@link Project} language(s) translations in one bundled zip. To get an
     * individual {@link Language} translation file's metadata, use
     * {@link #getProjectTranslatedFile(int, Language)}.
     * 
     * @param projectId
     *            ID of the {@link Project} to get the master file for
     * @return master translated {@link CloudwordsFile} resource
     * @throws CloudwordsClientException
     */
    public CloudwordsFile getMasterProjectTranslatedFile(int projectId) throws CloudwordsClientException;

    /**
     * Get the list of current {@link Project} {@link Language} translation files' metadata for the
     * specified Cloudwords {@link Project}. Each metadata representation in the list will be
     * specific to a {@link Project} target {@link Language} and contain a contentPath for
     * downloading the respective binary file.
     * 
     * @param projectId
     *            ID of the {@link Project} to get the translated files for
     * @return list of translated {@link Project} {@link CloudwordsLanguageFile} resources
     * @throws CloudwordsClientException
     */
    public List<CloudwordsLanguageFile> getProjectTranslatedFiles(int projectId) throws CloudwordsClientException;

    /**
     * Get the the current translated {@link Project} file metadata for the specified Cloudwords
     * {@link Project} and target {@link Language}. The returned metadata will contain a contentPath
     * for downloading the actual binary files.
     * 
     * @param projectId
     *            ID of the {@link Project} to get the translated file for
     * @param language
     *            The specific language
     * @return translated {@link Project} file for the specified {@link Language}
     * @throws CloudwordsClientException
     */
    public CloudwordsLanguageFile getProjectTranslatedFile(int projectId, Language language)
            throws CloudwordsClientException;

    /**
     * Download the current master translated material binary for the specified Cloudwords project
     * as an {@link InputStream}, provided that such translated material exists for the
     * {@link Project}. The master {@link Project} translation file contains all of the current
     * {@link Project} language(s) translation in one bundled zip. To download an individual
     * {@link Language} translation file, use {@link #downloadTranslatedFile(int, Language)}.
     * <p>
     * <b>NOTE:</b> clients utilizing this method must handle the closing of {@link InputStream}
     * after use.
     * </p>
     * 
     * @param projectId
     *            ID of the {@link Project} to download the master translated file for
     * @return file {@link InputStream} if reference file exists, null otherwise
     * @throws CloudwordsClientException
     */
    public InputStream downloadMasterTranslatedFile(int projectId) throws CloudwordsClientException;

    /**
     * Download the specified translated material binary for the specified Cloudwords
     * {@link Project} and target {@link Language} as an {@link InputStream}, provided that such
     * translated material exists for the {@link Project}.
     * <p>
     * <b>NOTE:</b> clients utilizing this method must handle the closing of {@link InputStream}
     * after use.
     * </p>
     * 
     * @param projectId
     *            ID of the {@link Project} to get the translated file for
     * @param language
     *            The specific {@link Language}
     * @return file {@link InputStream} if reference file exists, null otherwise
     * @throws CloudwordsClientException
     */
    public InputStream downloadTranslatedFile(int projectId, Language language) throws CloudwordsClientException;
    
    /**
     * Get the list of individual translated documents for a given language.
     * @param projectId ID of the {@link Project} to get the translated documents for
     * @param language The specific {@link Language}
     * @return The list of translated documents, which may be empty
     * @throws CloudwordsClientException
     */
    public List<TranslatedDocument> getTranslatedDocuments(int projectId, Language language) throws CloudwordsClientException;
    
    /**
     * Get the metadata for an individual translated document. To get all documents for a language, use {@link #getTranslatedDocuments(int, Language)}.
     * @param projectId ID of the {@link Project} to get the translated documents for
     * @param language The specific {@link Language}
     * @param documentId The ID of the translated document to fetch 
     * @return The document metadata
     * @throws CloudwordsClientException
     */
    public TranslatedDocument getTranslatedDocument(int projectId, Language language, int documentId) throws CloudwordsClientException;
    
    /**
     * Update the metadata of a translated document. Use this method to approve or request revision on an individual document.
     * @param projectId ID of the {@link Project} to get the translated documents for
     * @param language The specific {@link Language}
     * @param document The document metadata
     * @return The updated document
     * @throws CloudwordsClientException
     */
    public TranslatedDocument updateTranslatedDocument(int projectId, Language language, TranslatedDocument document) throws CloudwordsClientException;
    
    /**
     * Update the metadata for multiple translated documents simultaneously. This method can be used in the same way as {@link #updateTranslatedDocument(int, Language, TranslatedDocument)}.
     * Only documents with a status different from the server-side status will be affected. It is not required that all translated documents on the server be included.
     * @param projectId ID of the {@link Project} to update the translated documents for
     * @param language The specific {@link Language}
     * @param documents The translated documents
     * @return All translated documents for the language, with current status
     * @throws CloudwordsClientException
     */
    public List<TranslatedDocument> updateTranslatedDocuments(int projectId, Language language, List<TranslatedDocument> documents) throws CloudwordsClientException;
    
    /**
     * <p>
     * Upload a new translated document. In almost all cases, sourceId should be specified. However, certain projects may
     * have additional deliverables that are not the translation for a source document. In those cases, it is expected sourceId
     * will be null. The source ID should be derived from {@link SourceDocument#getId()} (SourceDocument is obtained by calling {@link #getSourceDocuments(int)}).
     * </p>
     * @param projectId ID of the {@link Project} to get the translated documents for
     * @param language The specific {@link Language}
     * @param sourceId The ID of the source document
     * @param file The translation to upload
     * @return The metadata for the newly created translated document.
     * @throws CloudwordsClientException
     */
    public TranslatedDocument addTranslatedDocument(int projectId, Language language, Integer sourceId, File file) throws CloudwordsClientException;
    
    /**
     * Replace a translated document. If the document has a pending revision request, this call will automatically close the request.
     * @param projectId The ID of the project
     * @param language The language of the translated document
     * @param documentId The translated document ID
     * @param file The new translation
     * @return The updated metadata of the translated document
     * @throws CloudwordsClientException
     */
    public TranslatedDocument replaceTranslatedDocument(int projectId, Language language, int documentId, File file) throws CloudwordsClientException;
    
    /**
     * Downloads the file made available via {@link TranslatedDocument#getFile()}. 
     * @param projectId The project ID
     * @param language The language of the translated document
     * @param documentId The ID of the translated document
     * @return An input stream for the bytes of the translation. The caller is expected to close the stream when done.
     */
    public InputStream downloadTranslatedDocumentFile(int projectId, Language language, int documentId) throws CloudwordsClientException, IllegalStateException, IOException;
    
    /**
     * Downloads the file made available via {@link TranslatedDocument#getXliff()}. 
     * @param projectId The project ID
     * @param language The language of the translated document
     * @param documentId The ID of the translated document
     * @return An input stream for the bytes of the translation. The caller is expected to close the stream when done.
     */
    public InputStream downloadTranslatedDocumentXLIFF(int projectId, Language language, int documentId) throws CloudwordsClientException, IllegalStateException, IOException;
    
    /**
     * Get the {@link Project}'s translation memory {@link CloudwordsFile} resource for the specified
     * Cloudwords. The metadata will contain a contentPath for downloading the actual binary file.
     * 
     * @param projectId
     *            ID of the {@link Project} to return translation memory for
     * @return {@link Project} translation memory {@link CloudwordsFile} resource
     * @throws CloudwordsClientException
     */
    public CloudwordsFile getProjectTranslationMemory(int projectId) throws CloudwordsClientException;
    
    /**
     * Download the current project translation memory for the specified Cloudwords project
     * as an {@link InputStream}, provided that such translated memory exists for the
     * {@link Project}.
     * <p>
     * <b>NOTE:</b> clients utilizing this method must handle the closing of {@link InputStream}
     * after use.
     * </p>
     * 
     * @param projectId
     *            ID of the {@link Project} to download the project translation memory file for
     * @return file {@link InputStream} if translation memory file exists, null otherwise
     * @throws CloudwordsClientException
     */
    public InputStream downloadProjectTranslationMemory(int projectId) throws CloudwordsClientException;
    
    /**
     * Approve the delivered {@link CloudwordsLanguageFile} for the specified Cloudwords
     * {@link Project} and target {@link Language}. 
     * 
     * @param projectId
     *            ID of the {@link Project} to get the translated file for
     * @param language
     *            The specific language
     * @return translated {@link Project} file for the specified {@link Language}
     * @throws CloudwordsClientException
     */
    public CloudwordsLanguageFile approveTranslatedFile(int projectId, Language language) throws CloudwordsClientException;

    /**
     * Convenience method to download the binary content of a {@link CloudwordsFile} as an
     * {@link InputStream}, provided that the contentPath inside the metadata is valid.
     * <p>
     * <b>NOTE:</b> clients utilizing this method must handle the closing of {@link InputStream}
     * after use.
     * </p>
     * 
     * @param metadata
     *            The specific file to download content for
     * @return file {@link InputStream} if the file exists, null otherwise
     * @throws CloudwordsClientException
     */
    public InputStream downloadFileFromMetadata(CloudwordsFile metadata) throws IllegalStateException,
            CloudwordsClientException, IOException;
    
    /**
     * Get the list of open {@link Project} in Cloudwords.
     * 
     * @return list of open customer {@link Project} resources
     * @deprecated use {@link #getProjects(PagedProjectQuery)}
     * @throws CloudwordsClientException
     */
    public List<Project> getOpenProjects() throws CloudwordsClientException;

    /**
     * Get the list of closed {@link Project} in Cloudwords.
     * 
     * @return list of closed customer {@link Project} resources
     * @deprecated use {@link #getProjects(PagedProjectQuery)}
     * @throws CloudwordsClientException
     */
    public List<Project> getClosedProjects() throws CloudwordsClientException;

    /**
     * Update the {@link Project} specified in the input {@link Project} id with the content
     * represented by the input. Please note that there are various restrictions as to what and when
     * a {@link Project} can be updated in Cloudwords. Please refer to API documentation for more
     * detail.
     * 
     * @param project
     *            The {@link Project} to update
     * @return updated {@link Project} resourcese
     * @throws CloudwordsClientException
     */
    public Project updateProject(Project project) throws CloudwordsClientException;
    
    /**
     * Get the {@link Task} attachment metadata, if it exists.
     * 
     * @param projectId ID of the {@link Project} for the associated task
     * @param taskId ID of the {@link Task} to return an attachment metatdata for
     * @return Metadata for the file uploaded
     * @throws CloudwordsClientException
     */
    public CloudwordsFile getTaskAttachment(int projectId, int taskId) throws CloudwordsClientException;
    
    /**
     * Download the {@link Task} attachment, if it exists.
     * 
     * @param taskId ID of the {@link Task} to return an attachment for
     * @return file {@link InputStream} if the file exists, null otherwise
     * @throws CloudwordsClientException
     */
    public InputStream downloadTaskAttachment(int projectId, int taskId) throws CloudwordsClientException;
    
    /**
     * Gets the {@link CustomerChangeOrder} attachment metadata, if it exists. Please refer to the API documentation for
     * more information.
     * @param projectId
     *              ID of the {@link Project} of the associated {@link CustomerChangeOrder}
     * @param changeOrderId
     *            ID of the {@link CustomerChangeOrder} to return an attachment metatdata for
     * @return Metadata for the file uploaded
     * @throws CloudwordsClientException
     */
    public CloudwordsFile getChangeOrderAttachment(int projectId, int changeOrderId) throws CloudwordsClientException;

    /**
     * Downloads the source material binary for the specified {@link CustomerChangeOrder} as an {@link InputStream},
     * provided that a source material exists for the {@link CustomerChangeOrder}. Please refer to the API documentation
     * for more information.
     * <p>
     * <b>NOTE:</b> clients utilizing this method must handle the closing of {@link InputStream} after use.
     * </p>
     * @param projectId
     *              ID of the {@link Project} of the associated {@link CustomerChangeOrder}
     * @param changeOrderId
     *            ID of {@link CustomerChangeOrder} to download source for
     * @return file {@link InputStream} if source file exists, null otherwise
     * @throws CloudwordsClientException
     */
    public InputStream downloadChangeOrderFile(int projectId, int changeOrderId) throws CloudwordsClientException;
    
    /**
     * Get the {@link ProjectWorkflow}s of the specified project. Workflows fetched through this method will have a
     * null value for {@link ProjectWorkflowPhase#getPossibleAssignees()}
     * @param projectId ID of the {@link Project} to fetch workflows for
     * @return A list of workflows, which may be empty if none have been specified.
     * @throws CloudwordsClientException
     */
    public List<ProjectWorkflow> getProjectWorkflows(int projectId) throws CloudwordsClientException;
    
    /**
     * Get a specific project workflow. Workflows fetched via this method will include possibleAssigness on the
     * phases where the assignee can be changed.
     * @param projectId The ID of the project to get a workflow for
     * @param languageCode The code of the target language to find a workflow for
     */
    public ProjectWorkflow getProjectWorkflow(int projectId, String languageCode) throws CloudwordsClientException;
    
    /**
     * Get the workflows for a change order. Change orders only have workflows if the Change Order is adding languages
     * to the project. The workflows are initialized automatically when the change order is created.  Workflows fetched 
     * through this method will have a null value for {@link ProjectWorkflowPhase#getPossibleAssignees()}
     * @param projectId The ID of the project
     * @param changeOrderId The ID of the change order
     * @throws CloudwordsClientException
     */
    public List<ChangeOrderProjectWorkflow> getChangeOrderProjectWorkflows(int projectId, int changeOrderId) throws CloudwordsClientException;
    
    /**
     * Get a specific change order project workflow. Workflows fetched via this method will include possibleAssigness 
     * on the phases where the assignee can be changed.
     * @param projectId The ID of the project
     * @param changeOrderId The ID of the change order
     * @param languageCode The code of the language to fetch the workflow for
     */
    public ChangeOrderProjectWorkflow getChangeOrderProjectWorkflow(int projectId, int changeOrderId, String languageCode) throws CloudwordsClientException;
    
    /**
     * Creates a new Cloudwords {@link Webhook}. Please refer to the API
     * documentation for more information.
     * 
     * @param eventType The eventType of the webhook
     * @param uri The uri of the webhook
     * @param entityType The entity type of the supplied entityId
     * @param entityId The entity ID for the webhook
     * @param contextType The context type of the supplied contextId
     * @param contextId The context ID for the webhook. 
     * @return {@link Webhook} created resource
     * @throws CloudwordsClientException
     */
    public Webhook createWebhook(String eventType, String uri, String entityType, Integer entityId, String contextType, String contextId) throws CloudwordsClientException;
    
    /**
     * Creates a new Cloudwords {@link Webhook}. Please refer to the API
     * documentation for more information.
     * 
     * @param eventType The eventType of the webhook
     * @param uri The uri of the webhook
     * @param project The project for the webhook
     * @return {@link Webhook} created resource
     * @throws CloudwordsClientException
     */
    public Webhook createWebhook(String eventType, String uri, Project project) throws CloudwordsClientException;    
    
    /**
     * Creates a new Cloudwords {@link Webhook}. Please refer to the API
     * documentation for more information.
     * 
     * @param eventType The eventType of the webhook
     * @param uri The uri of the webhook
     * @param project The project for the webhook
     * @param language The language for the webhook
     * @return {@link Webhook} created resource
     * @throws CloudwordsClientException
     */
    public Webhook createWebhook(String eventType, String uri, Project project, Language language) throws CloudwordsClientException;

    
    /**
     * Get a specific Cloudwords {@link Webhook}. Please refer to the API
     * documentation for more information.
     * 
     * @param webhookId
     *            ID of the {@link Webhook} to retrieve
     * @return {@link Webhook} specified resource
     * @throws CloudwordsClientException
     */
    public Webhook getWebhook(Integer webhookId) throws CloudwordsClientException;
    
    /**
     * Deletes an existing Cloudwords {@link Webhook}. Please refer to the API
     * documentation for more information.
     * 
     * @param webhookId
     *            ID of the {@link Webhook} to delete
     * @throws CloudwordsClientException
     */
    public void deleteWebhook(Integer webhookId) throws CloudwordsClientException;
    
    /**
     * Gets all Cloudwords {@link Webhook}s. Please refer to the API
     * documentation for more information.
     * 
     * @return List of existing {@link Webhook}s
     * @throws CloudwordsClientException
     */
    public List<Webhook> getWebhooks() throws CloudwordsClientException;
    
    /**
     * Search for projects and return them in a paged format. See API documentation for details.
     * 
     * @since 1.21
     * @throws CloudwordsClientException
     */
    public PagedResponse<Project> getProjects(PagedProjectQuery query) throws CloudwordsClientException;
}
