/*
 * Copyright 2011, Cloudwords, Inc.
 *
 * Licensed under the API LICENSE AGREEMENT, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.cloudwords.com/developers/license-1.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.cloudwords.api.client.resources;

import java.util.Date;

import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.apache.commons.lang3.builder.ToStringBuilder;

/**
 * Represents the metadata around a file stored in Cloudwords. The type of file could be the source
 * content for a {@link Project}, the reference material for a {@link Project}, etc.
 * 
 * @author jonathansu
 * @since 1.0
 */
public class CloudwordsFile {

    private Integer id;
    private String filename;
    private String contentPath;
    private Long sizeInKilobytes;
    private String fileContents;
    private Date createdDate;
    private String path;

    /**
     * Get the Cloudwords ID. This is the unique identifier for this resource.
     * 
     * @return The ID of this resource
     */
    public Integer getId() {
        return id;
    }

    /**
     * Set the Cloudwords ID. This is a read-only field with the Cloudwords API - settings
     * this and sending it to Cloudwords will have no effect.
     * 
     * @param id
     *            The ID of this resource
     */
    public void setId(Integer id) {
        this.id = id;
    }

    /**
     * Get the file name.
     * 
     * @return file name.
     */
    public String getFilename() {
        return filename;
    }

    /**
     * Set the file name.
     * 
     * @param filename
     */
    public void setFilename(String filename) {
        this.filename = filename;
    }

    /**
     * Get the API-accessible path to download the actual binary content of this file from
     * Cloudwords.
     * 
     * @return contentPath The URL to the binary content of this {@link CloudwordsFile}
     */
    public String getContentPath() {
        return contentPath;
    }

    /**
     * Set the content path. This is a read-only field with the Cloudwords API - settings this and
     * sending it to Cloudwords will have no effect.
     * 
     * @param contentPath
     *            The URL to the binary content of this {@link CloudwordsFile}
     */
    public void setContentPath(String contentPath) {
        this.contentPath = contentPath;
    }

    /**
     * Get the size of the binary content in kilobytes.
     * 
     * @return size in kilobytes
     */
    public Long getSizeInKilobytes() {
        return sizeInKilobytes;
    }

    /**
     * Set the size of the binary content in kilobytes. This is a read-only field with the
     * Cloudwords API - settings this and sending it to Cloudwords will have no effect.
     * 
     * @param sizeInKilobytes
     *            size in kilobytes
     */
    public void setSizeInKilobytes(Long sizeInKilobytes) {
        this.sizeInKilobytes = sizeInKilobytes;
    }

    /**
     * Get the String representation of the content inside the binary file.
     * 
     * @return fileContents The list of contents in the binary file, up to a certain number of
     *         characters
     */
    public String getFileContents() {
        return fileContents;
    }

    /**
     * Set the String representation of the content inside the binary file. This is a read-only
     * field with the Cloudwords API - settings this and sending it to Cloudwords will have no
     * effect.
     * 
     * @param fileContents
     *            The list of contents in the binary file, up to a certain number of characters
     */
    public void setFileContents(String fileContents) {
        this.fileContents = fileContents;
    }

    /**
     * Get the {@link CloudwordsFile} created date. Please note that the Date object contains the
     * full time, date and time zone information.
     * <p>
     * Please refer to the API documentation for more information on date handling.
     * </p>
     * 
     * @return createdDate The date this {@link CloudwordsFile} was created.
     */
    public Date getCreatedDate() {
        return createdDate;
    }

    /**
     * Set the {@link CloudwordsFile} created date. (Note: {@link CloudwordsFile} created date is a
     * read-only field with the Cloudwords API - setting it and sending it to the API will have no
     * effect.)
     * 
     * @param createdDate
     *            The date this {@link CloudwordsFile} was created.
     */
    public void setCreatedDate(Date createdDate) {
        this.createdDate = createdDate;
    }

    /**
     * Get the API-accessible path to this CloudwordsFile's metadata. This is the HTTP accessible
     * path to perform a GET on the {@link CloudwordsFile}. Please refer to the API documentation
     * for more detail.
     * 
     * @return path The URL to this {@link CloudwordsFile}
     */
    public String getPath() {
        return path;
    }

    /**
     * Set the API-accessible path to this CloudwordsFile's metadata. (Note: The path is a read-only
     * field with Cloudwords API - Cloudwords will automatically calculate the API-accessible path
     * on any {@link CloudwordsFile} representation returned from the API).
     * 
     * @param path
     *            The URL to this {@link CloudwordsFile}
     */
    public void setPath(String path) {
        this.path = path;
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this).append("id", id).append("filename", filename)
                .append("contentPath", contentPath).append("sizeInKilobytes", sizeInKilobytes)
                .append("fileContents", fileContents).append("createdDate", createdDate).append("path", path)
                .toString();
    }

    @Override
    public int hashCode() {
        return new HashCodeBuilder().append(id).append(filename).append(contentPath).append(sizeInKilobytes)
                .append(fileContents).append(createdDate).append(path).hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (obj == this) {
            return true;
        }
        if (obj.getClass() != getClass()) {
            return false;
        }
        CloudwordsFile rhs = (CloudwordsFile)obj;
        return new EqualsBuilder().append(id, rhs.id).append(filename, rhs.filename)
                .append(contentPath, rhs.contentPath).append(sizeInKilobytes, rhs.sizeInKilobytes)
                .append(fileContents, rhs.fileContents).append(createdDate, rhs.createdDate).append(path, rhs.path)
                .isEquals();
    }
}
